@@ -111,12 +111,6 @@ void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_f
 				}
 			}
 			return;
-		case fz::socket_event_flag::close:
-			{
-				controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), fz::socket_error_description(error));
-				TransferEnd(TransferEndReason::failure);
-			}
-			return;
 		default:
 			// Uninteresting
 			break;
@@ -138,22 +132,28 @@ void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_f
 	{
 	case fz::socket_event_flag::connection:
 		if (error) {
-			if (m_transferEndReason == TransferEndReason::none) {
-				controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), fz::socket_error_description(error));
-				TransferEnd(TransferEndReason::transfer_failure);
-			}
+			controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), fz::socket_error_description(error));
+			TransferEnd(TransferEndReason::transfer_failure);
 		}
-		else
+		else {
 			OnConnect();
+		}
 		break;
 	case fz::socket_event_flag::read:
-		OnReceive();
+		if (error) {
+			OnSocketError(error);
+		}
+		else {
+			OnReceive();
+		}
 		break;
 	case fz::socket_event_flag::write:
-		OnSend();
-		break;
-	case fz::socket_event_flag::close:
-		OnClose(error);
+		if (error) {
+			OnSocketError(error);
+		}
+		else {
+			OnSend();
+		}
 		break;
 	default:
 		// Uninteresting
@@ -232,98 +232,98 @@ void CTransferSocket::OnReceive()
 		return;
 	}
 
-	if (m_transferMode == TransferMode::list) {
-		for (;;) {
-			char *pBuffer = new char[4096];
-			int error;
-			int numread = m_pBackend->Read(pBuffer, 4096, error);
-			if (numread < 0) {
-				delete [] pBuffer;
-				if (error != EAGAIN) {
-					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
-					TransferEnd(TransferEndReason::transfer_failure);
-				}
-				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
-					TransferEnd(TransferEndReason::successful);
-				}
-				return;
-			}
-
-			if (numread > 0) {
-				if (!m_pDirectoryListingParser->AddData(pBuffer, numread)) {
-					TransferEnd(TransferEndReason::transfer_failure);
+	if (m_transferEndReason == TransferEndReason::none) {
+		if (m_transferMode == TransferMode::list) {
+			for (;;) {
+				char *pBuffer = new char[4096];
+				int error;
+				int numread = m_pBackend->Read(pBuffer, 4096, error);
+				if (numread < 0) {
+					delete [] pBuffer;
+					if (error != EAGAIN) {
+						controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
+						TransferEnd(TransferEndReason::transfer_failure);
+					}
 					return;
 				}
 
+				if (numread > 0) {
+					if (!m_pDirectoryListingParser->AddData(pBuffer, numread)) {
+						TransferEnd(TransferEndReason::transfer_failure);
+						return;
+					}
+
+					controlSocket_.SetActive(CFileZillaEngine::recv);
+					if (!m_madeProgress) {
+						m_madeProgress = 2;
+						engine_.transfer_status_.SetMadeProgress();
+					}
+					engine_.transfer_status_.Update(numread);
+				}
+				else {
+					delete [] pBuffer;
+					TransferEnd(TransferEndReason::successful);
+					return;
+				}
+			}
+			return;
+		}
+		else if (m_transferMode == TransferMode::download) {
+			int error;
+			int numread;
+
+			// Only do a certain number of iterations in one go to keep the event loop going.
+			// Otherwise this behaves like a livelock on very large files written to a very fast
+			// SSD downloaded from a very fast server.
+			for (int i = 0; i < 100; ++i) {
+				if (!CheckGetNextWriteBuffer()) {
+					return;
+				}
+
+				numread = m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen, error);
+				if (numread <= 0) {
+					break;
+				}
+
 				controlSocket_.SetActive(CFileZillaEngine::recv);
 				if (!m_madeProgress) {
 					m_madeProgress = 2;
 					engine_.transfer_status_.SetMadeProgress();
 				}
 				engine_.transfer_status_.Update(numread);
-			}
-			else {
-				delete [] pBuffer;
-				TransferEnd(TransferEndReason::successful);
-				return;
-			}
-		}
-	}
-	else if (m_transferMode == TransferMode::download) {
-		int error;
-		int numread;
 
-		// Only do a certain number of iterations in one go to keep the event loop going.
-		// Otherwise this behaves like a livelock on very large files written to a very fast
-		// SSD downloaded from a very fast server.
-		for (int i = 0; i < 100; ++i) {
-			if (!CheckGetNextWriteBuffer()) {
-				return;
+				m_pTransferBuffer += numread;
+				m_transferBufferLen -= numread;
 			}
 
-			numread = m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen, error);
-			if (numread <= 0) {
-				break;
-			}
-
-			controlSocket_.SetActive(CFileZillaEngine::recv);
-			if (!m_madeProgress) {
-				m_madeProgress = 2;
-				engine_.transfer_status_.SetMadeProgress();
-			}
-			engine_.transfer_status_.Update(numread);
-
-			m_pTransferBuffer += numread;
-			m_transferBufferLen -= numread;
-		}
-
-		if (numread < 0) {
-			if (error != EAGAIN) {
-				controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
-				TransferEnd(TransferEndReason::transfer_failure);
-			}
-			else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
-				FinalizeWrite();
-			}
-		}
-		else if (!numread) {
-			FinalizeWrite();
-		}
-		else {
-			send_event<fz::socket_event>(m_pBackend, fz::socket_event_flag::read, 0);
-		}
-	}
-	else if (m_transferMode == TransferMode::resumetest) {
-		for (;;) {
-			char buffer[2];
-			int error;
-			int numread = m_pBackend->Read(buffer, 2, error);
 			if (numread < 0) {
 				if (error != EAGAIN) {
 					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
-				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
+			}
+			else if (!numread) {
+				FinalizeWrite();
+			}
+			else {
+				send_event<fz::socket_event>(m_pBackend, fz::socket_event_flag::read, 0);
+			}
+			return;
+		}
+		else if (m_transferMode == TransferMode::resumetest) {
+			for (;;) {
+				char buffer[2];
+				int error;
+				int numread = m_pBackend->Read(buffer, 2, error);
+				if (numread < 0) {
+					if (error != EAGAIN) {
+						controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
+						TransferEnd(TransferEndReason::transfer_failure);
+					}
+					return;
+				}
+
+				if (!numread) {
 					if (m_transferBufferLen == 1) {
 						TransferEnd(TransferEndReason::successful);
 					}
@@ -331,27 +331,39 @@ void CTransferSocket::OnReceive()
 						controlSocket_.LogMessage(MessageType::Debug_Warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
 						TransferEnd(TransferEndReason::failed_resumetest);
 					}
+					return;
 				}
-				return;
-			}
+				m_transferBufferLen += numread;
 
-			if (!numread) {
-				if (m_transferBufferLen == 1) {
-					TransferEnd(TransferEndReason::successful);
-				}
-				else {
+				if (m_transferBufferLen > 1) {
 					controlSocket_.LogMessage(MessageType::Debug_Warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
 					TransferEnd(TransferEndReason::failed_resumetest);
+					return;
 				}
-				return;
 			}
-			m_transferBufferLen += numread;
+			return;
+		}
+	}
 
-			if (m_transferBufferLen > 1) {
-				controlSocket_.LogMessage(MessageType::Debug_Warning, L"Server incorrectly sent %d bytes", m_transferBufferLen);
-				TransferEnd(TransferEndReason::failed_resumetest);
-				return;
-			}
+	char discard[1024];
+	int error;
+	int numread = m_pBackend->Read(discard, 1024, error);
+
+	if (m_transferEndReason == TransferEndReason::none) {
+		// If we get here we're uploading
+
+		if (numread > 0) {
+			controlSocket_.LogMessage(MessageType::Error, L"Received data from the server during an upload");
+			TransferEnd(TransferEndReason::transfer_failure);
+		}
+		else if (numread < 0 && error != EAGAIN) {
+			controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
+			TransferEnd(TransferEndReason::transfer_failure);
+		}
+	}
+	else {
+		if (!numread || (numread < 0 && error != EAGAIN)) {
+			ResetSocket();
 		}
 	}
 }
@@ -369,7 +381,7 @@ void CTransferSocket::OnSend()
 		return;
 	}
 
-	if (m_transferMode != TransferMode::upload) {
+	if (m_transferMode != TransferMode::upload || m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
@@ -419,77 +431,16 @@ void CTransferSocket::OnSend()
 	}
 }
 
-void CTransferSocket::OnClose(int error)
+void CTransferSocket::OnSocketError(int error)
 {
-	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnClose(%d)", error);
-	m_onCloseCalled = true;
+	controlSocket_.LogMessage(MessageType::Debug_Verbose, L"CTransferSocket::OnSocketError(%d)", error);
 
 	if (m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
-	if (!m_pBackend) {
-		if (!InitBackend()) {
-			TransferEnd(TransferEndReason::transfer_failure);
-			return;
-		}
-	}
-
-	if (m_transferMode == TransferMode::upload) {
-		if (m_shutdown && m_pTlsSocket) {
-			if (m_pTlsSocket->Shutdown() != 0) {
-				TransferEnd(TransferEndReason::transfer_failure);
-			}
-			else {
-				TransferEnd(TransferEndReason::successful);
-			}
-		}
-		else {
-			TransferEnd(TransferEndReason::transfer_failure);
-		}
-		return;
-	}
-
-	if (error) {
-		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
-		TransferEnd(TransferEndReason::transfer_failure);
-		return;
-	}
-
-	char buffer[100];
-	int numread = m_pBackend->Peek(&buffer, 100, error);
-	if (numread > 0) {
-#ifndef FZ_WINDOWS
-		controlSocket_.LogMessage(MessageType::Debug_Warning, L"Peek isn't supposed to return data after close notification");
-#endif
-
-		// MSDN says this:
-		//   FD_CLOSE being posted after all data is read from a socket.
-		//   An application should check for remaining data upon receipt
-		//   of FD_CLOSE to avoid any possibility of losing data.
-		// First half is actually plain wrong.
-		OnReceive();
-
-		return;
-	}
-	else if (numread < 0 && error != EAGAIN) {
-		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
-		TransferEnd(TransferEndReason::transfer_failure);
-		return;
-	}
-
-	if (m_transferMode == TransferMode::resumetest) {
-		if (m_transferBufferLen != 1) {
-			TransferEnd(TransferEndReason::failed_resumetest);
-			return;
-		}
-	}
-	if (m_transferMode == TransferMode::download) {
-		FinalizeWrite();
-	}
-	else {
-		TransferEnd(TransferEndReason::successful);
-	}
+	controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
+	TransferEnd(TransferEndReason::transfer_failure);
 }
 
 bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
@@ -574,7 +525,7 @@ void CTransferSocket::SetActive()
 		return;
 	}
 
-	if (socket_->get_state() == fz::socket::connected || socket_->get_state() == fz::socket::closing) {
+	if (socket_->get_state() == fz::socket::connected) {
 		TriggerPostponedEvents();
 	}
 }
@@ -588,7 +539,9 @@ void CTransferSocket::TransferEnd(TransferEndReason reason)
 	}
 	m_transferEndReason = reason;
 
-	ResetSocket();
+	if (reason != TransferEndReason::successful) {
+		ResetSocket();
+	}
 
 	controlSocket_.send_event<TransferEndEvent>();
 }
@@ -694,9 +647,7 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 		}
 		else if (res == IO_Success) {
 			if (m_pTlsSocket) {
-				m_shutdown = true;
-
-				int error = m_pTlsSocket->Shutdown();
+				int error = m_pTlsSocket->Shutdown(true);
 				if (error != 0) {
 					if (error != EAGAIN) {
 						TransferEnd(TransferEndReason::transfer_failure);
@@ -704,6 +655,7 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 					return false;
 				}
 			}
+			socket_->shutdown();
 			TransferEnd(TransferEndReason::successful);
 			return false;
 		}
@@ -759,12 +711,6 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 	assert(!m_pBackend);
 	m_pTlsSocket = new CTlsSocket(this, *socket_, &controlSocket_);
 
-	if (!m_pTlsSocket->Init()) {
-		delete m_pTlsSocket;
-		m_pTlsSocket = nullptr;
-		return false;
-	}
-
 	bool try_resume = CServerCapabilities::GetCapability(controlSocket_.currentServer_, tls_resume) != no;
 
 	int res = m_pTlsSocket->Handshake(pPrimaryTlsSocket, try_resume);
@@ -799,9 +745,6 @@ void CTransferSocket::TriggerPostponedEvents()
 			return;
 		}
 	}
-	if (m_onCloseCalled) {
-		OnClose(0);
-	}
 }
 
 bool CTransferSocket::InitBackend()
