@@ -47,17 +47,17 @@ bool CLocalRecursiveOperation::DoStartRecursiveOperation(OperationMode mode, Act
 	if (!m_pQueue) {
 		return false;
 	}
-	
-	auto const server = m_state.GetServer();
-	if (server) {
-		server_ = *server;
+
+	Site const& site = m_state.GetSite();
+	if (site) {
+		site_ = site;
 	}
 	else {
 		if (mode != OperationMode::recursive_list) {
 			return false;
 		}
 
-		server_ = CServer();
+		site_ = Site();
 	}
 
 	{
@@ -121,7 +121,7 @@ void CLocalRecursiveOperation::StopRecursiveOperation()
 	m_actionAfterBlocker.reset();
 }
 
-void CLocalRecursiveOperation::OnStateChange(t_statechange_notifications, const wxString&, const void*)
+void CLocalRecursiveOperation::OnStateChange(t_statechange_notifications, std::wstring const&, const void*)
 {
 }
 
@@ -167,8 +167,6 @@ void CLocalRecursiveOperation::entry()
 		auto filters = m_filters.first;
 
 		CFilterManager filterManager;
-		filterManager.CompileRegexes(filters);
-
 		while (!recursion_roots_.empty()) {
 			listing d;
 
@@ -279,18 +277,18 @@ void CLocalRecursiveOperation::OnListedDirectory()
 		}
 		else {
 			if (queue) {
-				m_pQueue->QueueFiles(!m_immediate, server_, d);
+				m_pQueue->QueueFiles(!m_immediate, site_, d);
 			}
 			++m_processedDirectories;
 			processed += d.files.size();
-			m_state.NotifyHandlers(STATECHANGE_LOCAL_RECURSION_LISTING, wxString(), &d);
+			m_state.NotifyHandlers(STATECHANGE_LOCAL_RECURSION_LISTING, std::wstring(), &d);
 		}
 	}
 
 	if (queue) {
 		m_pQueue->QueueFile_Finish(m_immediate);
 	}
-	
+
 	m_processedFiles += processed;
 	if (stop) {
 		StopRecursiveOperation();
