@@ -303,7 +303,7 @@ void CMenuBar::UpdateBookmarkMenu()
 	auto ids = m_bookmark_menu_ids.begin();
 
 	// Insert global bookmarks
-	std::vector<wxString> global_bookmarks;
+	std::vector<std::wstring> global_bookmarks;
 	if (CBookmarksDialog::GetGlobalBookmarks(global_bookmarks) && !global_bookmarks.empty()) {
 		pMenu->AppendSeparator();
 
@@ -318,8 +318,7 @@ void CMenuBar::UpdateBookmarkMenu()
 				id = *ids;
 				++ids;
 			}
-			wxString name(bookmark);
-			name.Replace(_T("&"), _T("&&"));
+			std::wstring name(LabelEscape(bookmark));
 			pMenu->Append(id, name);
 
 			m_bookmark_menu_id_map_global[id] = bookmark;
@@ -335,7 +334,7 @@ void CMenuBar::UpdateBookmarkMenu()
 	}
 
 	Site site = pState->GetSite();
-	if (!site.server_) {
+	if (!site) {
 		site = pState->GetLastSite();
 	}
 	if (site.m_bookmarks.empty()) {
@@ -355,8 +354,7 @@ void CMenuBar::UpdateBookmarkMenu()
 			id = *ids;
 			++ids;
 		}
-		wxString name(bookmark.m_name);
-		name.Replace(_T("&"), _T("&&"));
+		wxString name(LabelEscape(bookmark.m_name));
 		pMenu->Append(id, name);
 
 		m_bookmark_menu_id_map_site[id] = bookmark.m_name;
@@ -381,7 +379,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 		}
 
 		Site site = pState->GetSite();
-		if (!site.server_) {
+		if (!site) {
 			site = pState->GetLastSite();
 		}
 
@@ -391,8 +389,8 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 
 				pState->SetSyncBrowse(false);
 				if (!bookmark.m_remoteDir.empty() && pState->IsRemoteIdle(true)) {
-					ServerWithCredentials const& server = pState->GetServer();
-					if (!server || server != site.server_) {
+					Site const& activeSite = pState->GetSite();
+					if (!activeSite || activeSite != site) {
 						m_pMainFrame->ConnectToSite(site, bookmark);
 						break;
 					}
@@ -401,7 +399,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 					}
 				}
 				if (!bookmark.m_localDir.empty()) {
-					bool set = pState->SetLocalDir(bookmark.m_localDir.ToStdWstring());
+					bool set = pState->SetLocalDir(bookmark.m_localDir);
 
 					if (set && bookmark.m_sync) {
 						wxASSERT(!bookmark.m_remoteDir.empty());
@@ -433,8 +431,8 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 
 		pState->SetSyncBrowse(false);
 		if (!remote_dir.empty() && pState->IsRemoteIdle(true)) {
-			ServerWithCredentials const& server = pState->GetServer();
-			if (server) {
+			Site const& activeSite = pState->GetSite();
+			if (activeSite) {
 				CServerPath current_remote_path = pState->GetRemotePath();
 				if (!current_remote_path.empty() && current_remote_path.GetType() != remote_dir.GetType()) {
 					wxMessageBoxEx(_("Selected global bookmark and current server use a different server type.\nUse site-specific bookmarks for this server."), _("Bookmark"), wxICON_EXCLAMATION, this);
@@ -462,7 +460,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 	event.Skip();
 }
 
-void CMenuBar::OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
+void CMenuBar::OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*)
 {
 	switch (notification)
 	{
@@ -584,30 +582,30 @@ void CMenuBar::UpdateMenubarState()
 		return;
 	}
 
-	ServerWithCredentials const& server = pState->GetServer();
+	Site const& site = pState->GetSite();
 	const bool idle = pState->IsRemoteIdle();
 
-	Enable(XRCID("ID_MENU_SERVER_DISCONNECT"), server && idle);
-	Enable(XRCID("ID_CANCEL"), server && !idle);
-	Enable(XRCID("ID_MENU_SERVER_CMD"), server && idle);
-	Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), server.operator bool());
-	Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), server.operator bool());
+	Enable(XRCID("ID_MENU_SERVER_DISCONNECT"), site && idle);
+	Enable(XRCID("ID_CANCEL"), site && !idle);
+	Enable(XRCID("ID_MENU_SERVER_CMD"), site && idle);
+	Enable(XRCID("ID_MENU_FILE_COPYSITEMANAGER"), site.operator bool());
+	Enable(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), site.operator bool());
 
 	Check(XRCID("ID_TOOLBAR_COMPARISON"), pState->GetComparisonManager()->IsComparing());
 	Check(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pState->GetSyncBrowse());
 
 	bool canReconnect;
-	if (server || !idle) {
+	if (site || !idle) {
 		canReconnect = false;
 	}
 	else {
-		canReconnect = static_cast<bool>(pState->GetLastSite().server_);
+		canReconnect = static_cast<bool>(pState->GetLastSite());
 	}
 	Enable(XRCID("ID_MENU_SERVER_RECONNECT"), canReconnect);
 
-	Enable(XRCID("ID_MENU_TRANSFER_TYPE"), !server || CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::DataTypeConcept));
-	Enable(XRCID("ID_MENU_TRANSFER_PRESERVETIMES"), !server || CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::PreserveTimestamp));
-	Enable(XRCID("ID_MENU_SERVER_CMD"), !server || CServer::ProtocolHasFeature(server.server.GetProtocol(), ProtocolFeature::EnterCommand));
+	Enable(XRCID("ID_MENU_TRANSFER_TYPE"), !site || CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::DataTypeConcept));
+	Enable(XRCID("ID_MENU_TRANSFER_PRESERVETIMES"), !site || CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::PreserveTimestamp));
+	Enable(XRCID("ID_MENU_SERVER_CMD"), !site || CServer::ProtocolHasFeature(site.server.GetProtocol(), ProtocolFeature::EnterCommand));
 }
 
 bool CMenuBar::ShowItem(int id)
