@@ -5,42 +5,6 @@
 
 #include "xmlfunctions.h"
 
-class Bookmark final
-{
-public:
-	bool operator==(Bookmark const& b) const;
-	bool operator!=(Bookmark const& b) const { return !(*this == b); }
-
-	wxString m_localDir;
-	CServerPath m_remoteDir;
-
-	bool m_sync{};
-	bool m_comparison{};
-
-	wxString m_name;
-};
-
-class Site final
-{
-public:
-	explicit operator bool() const { return server_.operator bool(); }
-
-	bool empty() const { return !server_; }
-	bool operator==(Site const& s) const;
-	bool operator!=(Site const& s) const { return !(*this == s); }
-
-	ServerWithCredentials server_;
-	wxString m_comments;
-
-	Bookmark m_default_bookmark;
-
-	std::vector<Bookmark> m_bookmarks;
-
-	std::wstring m_path;
-
-	wxColour m_colour;
-};
-
 class CSiteManagerXmlHandler
 {
 public:
@@ -68,7 +32,7 @@ public:
 
 	static std::pair<std::unique_ptr<Site>, Bookmark> GetSiteByPath(std::wstring const& sitePath, bool printErrors = true);
 
-	static wxString AddServer(ServerWithCredentials server);
+	static std::wstring AddServer(Site site);
 	static bool AddBookmark(std::wstring sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync, bool comparison);
 	static bool ClearBookmarks(std::wstring sitePath);
 
